@@ -15,7 +15,11 @@
 # limitations under the License.
 # This represents the low layer message framing portion of IPMI
 
+import pyghmi.constants as const
+import pyghmi.exceptions as exc
+
 from pyghmi.ipmi.private import session
+import pyghmi.ipmi.sdr as sdr
 
 
 boot_devices = {
@@ -83,6 +87,9 @@ class Command(object):
                                                 onlogon=self.logged,
                                                 port=port,
                                                 kg=kg)
+            # induce one iteration of the loop, now that we would be
+            # prepared for it in theory
+            session.Session.wait_for_rsp(0)
         else:
             self.ipmi_session = session.Session(bmc=bmc,
                                                 userid=userid,
@@ -98,6 +105,16 @@ class Command(object):
         while (session.Session.wait_for_rsp()):
             pass
 
+    @classmethod
+    def wait_for_rsp(cls, timeout):
+        """Delay for no longer than timeout for next response.
+
+        This acts like a sleep that exits on activity.
+
+        :param timeout: Maximum number of seconds before returning
+        """
+        return session.Session.wait_for_rsp(timeout=timeout)
+
     def get_bootdev(self):
         """Get current boot device override information.
 
@@ -108,9 +125,7 @@ class Command(object):
 
         :returns: dict --The response will be provided in the return as a dict
         """
-        response = self.ipmi_session.raw_command(netfn=0,
-                                                 command=9,
-                                                 data=(5, 0, 0))
+        response = self.raw_command(netfn=0, command=9, data=(5, 0, 0))
         # interpret response per 'get system boot options'
         if 'error' in response:
             return response
@@ -150,20 +165,21 @@ class Command(object):
         :returns: dict -- A dict describing the response retrieved
         """
         if powerstate not in power_states:
-            raise Exception("Unknown power state %s requested" % powerstate)
+            raise exc.InvalidParameterValue(
+                "Unknown power state %s requested" % powerstate)
         self.newpowerstate = powerstate
-        response = self.ipmi_session.raw_command(netfn=0, command=1)
+        response = self.raw_command(netfn=0, command=1)
         if 'error' in response:
-            raise Exception(response['error'])
+            raise exc.IpmiException(response['error'])
         self.powerstate = 'on' if (response['data'][0] & 1) else 'off'
         if self.powerstate == self.newpowerstate:
             return {'powerstate': self.powerstate}
         if self.newpowerstate == 'boot':
             self.newpowerstate = 'on' if self.powerstate == 'off' else 'reset'
-        response = self.ipmi_session.raw_command(
+        response = self.raw_command(
             netfn=0, command=2, data=[power_states[self.newpowerstate]])
         if 'error' in response:
-            raise Exception(response['error'])
+            raise exc.IpmiException(response['error'])
         self.lastresponse = {'pendingpowerstate': self.newpowerstate}
         waitattempts = 300
         if not isinstance(wait, bool):
@@ -176,14 +192,14 @@ class Command(object):
                 self.waitpowerstate = self.newpowerstate
             currpowerstate = None
             while currpowerstate != self.waitpowerstate and waitattempts > 0:
-                response = self.ipmi_session.raw_command(netfn=0, command=1,
-                                                         delay_xmit=1)
+                response = self.raw_command(netfn=0, command=1, delay_xmit=1)
                 if 'error' in response:
                     return response
                 currpowerstate = 'on' if (response['data'][0] & 1) else 'off'
                 waitattempts -= 1
             if currpowerstate != self.waitpowerstate:
-                raise Exception("System did not accomplish power state change")
+                raise exc.IpmiException(
+                    "System did not accomplish power state change")
             return {'powerstate': currpowerstate}
         else:
             return self.lastresponse
@@ -227,8 +243,7 @@ class Command(object):
         # then move on to set chassis capabilities
         self.requestpending = True
         # Set System Boot Options is netfn=0, command=8, data
-        response = self.ipmi_session.raw_command(netfn=0, command=8,
-                                                 data=(3, 8))
+        response = self.raw_command(netfn=0, command=8, data=(3, 8))
         self.lastresponse = response
         if 'error' in response:
             return response
@@ -240,12 +255,13 @@ class Command(object):
         if self.bootdev == 0:
             bootflags = 0
         data = (5, bootflags, self.bootdev, 0, 0, 0)
-        response = self.ipmi_session.raw_command(netfn=0, command=8, data=data)
+        response = self.raw_command(netfn=0, command=8, data=data)
         if 'error' in response:
             return response
         return {'bootdev': bootdev}
 
-    def raw_command(self, netfn, command, data=()):
+    def raw_command(self, netfn, command, bridge_request={}, data=(),
+                    delay_xmit=None):
         """Send raw ipmi command to BMC
 
         This allows arbitrary IPMI bytes to be issued.  This is commonly used
@@ -255,11 +271,14 @@ class Command(object):
 
         :param netfn: Net function number
         :param command: Command value
+        :param bridge_request: The target slave address and channel number for
+                               the bridge request.
         :param data: Command data as a tuple or list
         :returns: dict -- The response from IPMI device
         """
         return self.ipmi_session.raw_command(netfn=netfn, command=command,
-                                             data=data)
+                                             bridge_request=bridge_request,
+                                             data=data, delay_xmit=delay_xmit)
 
     def get_power(self):
         """Get current power state of the managed system
@@ -269,10 +288,43 @@ class Command(object):
 
         :returns: dict -- {'powerstate': value}
         """
-        response = self.ipmi_session.raw_command(netfn=0, command=1)
+        response = self.raw_command(netfn=0, command=1)
         if 'error' in response:
-            raise Exception(response['error'])
-            return
+            raise exc.IpmiException(response['error'])
         assert(response['command'] == 1 and response['netfn'] == 1)
         self.powerstate = 'on' if (response['data'][0] & 1) else 'off'
         return {'powerstate': self.powerstate}
+
+    def get_health(self):
+        """Summarize health of managed system
+
+        This provides a summary of the health of the managed system.
+        It additionally provides an iterable list of reasons for
+        warning, critical, or failed assessments.
+        """
+        summary = {}
+        summary['badreadings'] = []
+        summary['health'] = const.Health.Ok
+        for reading in self.get_sensor_data():
+            if reading.health != const.Health.Ok:
+                summary['health'] |= reading.health
+                summary['badreadings'].append(reading)
+        return summary
+
+    def get_sensor_data(self):
+        """Get sensor reading objects
+
+        Iterates sensor reading objects pertaining to the currently
+        managed BMC.
+
+        :returns: Iterator of sdr.SensorReading objects
+        """
+        if '_sdr' not in self.__dict__:
+            self._sdr = sdr.SDR(self)
+        for sensor in self._sdr.get_sensor_numbers():
+            rsp = self.raw_command(command=0x2d, netfn=4, data=(sensor,))
+            if 'error' in rsp:
+                if rsp['code'] == 203:  # Sensor does not exist, optional dev
+                    continue
+                raise Exception(rsp['error'])
+            yield self._sdr.sensors[sensor].decode_sensor_reading(rsp['data'])
